# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2017-09-04 09:13
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Consumer',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('key', models.CharField(max_length=128, unique=True, verbose_name='Key')),
                ('secret', models.CharField(max_length=128, unique=True, verbose_name='Secret')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='User')),
            ],
            options={
                'verbose_name': 'Consumer',
                'verbose_name_plural': 'Consumers',
            },
        ),
        migrations.CreateModel(
            name='TimestampAndNonce',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('timestamp', models.IntegerField(verbose_name='Timestamp')),
                ('nonce', models.CharField(max_length=128, verbose_name='Nonce')),
                ('consumer', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='lti_provider.Consumer', verbose_name='Consumer')),
            ],
            options={
                'verbose_name': 'Timestamp and Nonce',
                'verbose_name_plural': 'Timestamps and Nonces',
            },
        ),
        migrations.AlterUniqueTogether(
            name='timestampandnonce',
            unique_together=set([('timestamp', 'nonce')]),
        ),
    ]




# Copyright (c) 2018 Josef Wachtler
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""
This module provides validators for model fields of the lti_provider-App.
"""

import logging
from oauthlib.oauth1.rfc5849.utils import UNICODE_ASCII_CHARACTER_SET

from django.core.exceptions import ValidationError
from django.utils.translation import ugettext_lazy as _

# Get an instance of a logger
logger = logging.getLogger('LTI.lti_provider')


def validate_oauth_chars(value):
    """
    Validates if there are only unicode ascii characters.

    Keyword arguments:
        - value -- string to validate
    """
    a = set(UNICODE_ASCII_CHARACTER_SET)
    v = set(value)
    if not v <= a:
        logger.debug('unsave characters')
        raise ValidationError(
            _('%(value)s contains unsave characters.'),
            params={'value': value},
        )


def validate_oauth_length(value):
    """
    Validates if the length is between 20 and 30.

    Keyword arguments:
        - value -- string to validate
    """
    length = len(value)
    if length > 30:
        logger.debug('too long')
        raise ValidationError(
            _('%(value)s characters are too much. Max length: 30'),
            params={'value': str(length)},
        )
    if length < 20:
        logger.debug('too short')
        raise ValidationError(
            _('%(value)s characters are too few. Min length: 20'),
            params={'value': str(length)},
        )




# Copyright (c) 2018 Josef Wachtler
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""
This module provides all Django-Database-Models of the lti_provider-App.
"""

from django.db import models
from django.conf import settings
from django.utils.translation import ugettext_lazy as _
from lti_provider.field_validators import validate_oauth_chars, \
                                          validate_oauth_length


class Consumer(models.Model):
    """
    This model saves information about the consumers.

    Fields:
        - key -- a key to identify a consumer
        - secret -- the key of the consumer
        - user -- a user responsible for a consumer
    """
    key = models.CharField(max_length=128, unique=True,
                           verbose_name=_('Key'),
                           validators=[validate_oauth_chars,
                                       validate_oauth_length])
    secret = models.CharField(max_length=128, unique=True,
                              verbose_name=_('Secret'),
                              validators=[validate_oauth_chars,
                                          validate_oauth_length])
    user = models.ForeignKey(settings.AUTH_USER_MODEL,
                             on_delete=models.CASCADE,
                             verbose_name=_('User'))

    def __str__(self):
        """
        unicode representation
        """
        return str(self.key)

    class Meta:
        verbose_name = _('Consumer')
        verbose_name_plural = _('Consumers')


class TimestampAndNonce(models.Model):
    """
    This model stores the timestamp and its related nonce of a request
    from a consumer.

    Fields:
        - consumer -- the consumer of the request
        - timestamp -- time of the request as int
        - nonce -- a string related to this timestamp only
    """
    consumer = models.ForeignKey(Consumer,
                                 on_delete=models.CASCADE,
                                 verbose_name=_('Consumer'))

    timestamp = models.IntegerField(verbose_name=_('Timestamp'))
    nonce = models.CharField(max_length=128, verbose_name=_('Nonce'))

    def __str__(self):
        """
        unicode representation
        """
        return str(self.timestamp)

    class Meta:
        unique_together = ('timestamp', 'nonce')
        verbose_name = _('Timestamp and Nonce')
        verbose_name_plural = _('Timestamps and Nonces')
